@@ -66,6 +66,41 @@ function main() {
 
     // Handle the registered mime extensions.
     var mimeExtensions = [];
+    try {
+        if (isDeferred('')) {
+            deferred.matches.push('');
+            ignorePlugins.push('');
+        }
+        if (isDisabled('@jupyterlab/json-extension')) {
+            disabled.matches.push('@jupyterlab/json-extension');
+        } else {
+            var module = require('@jupyterlab/json-extension/');
+            var extension = module.default;
+
+            // Handle CommonJS exports.
+            if (!module.hasOwnProperty('__esModule')) {
+              extension = module;
+            }
+
+            if (Array.isArray(extension)) {
+                extension.forEach(function(plugin) {
+                    if (isDeferred(plugin.id)) {
+                        deferred.matches.push(plugin.id);
+                        ignorePlugins.push(plugin.id);
+                    }
+                    if (isDisabled(plugin.id)) {
+                        disabled.matches.push(plugin.id);
+                        return;
+                    }
+                    mimeExtensions.push(plugin);
+                });
+            } else {
+                mimeExtensions.push(extension);
+            }
+        }
+    } catch (e) {
+        console.error(e);
+    }
     try {
         if (isDeferred('')) {
             deferred.matches.push('');
@@ -101,6 +136,41 @@ function main() {
     } catch (e) {
         console.error(e);
     }
+    try {
+        if (isDeferred('')) {
+            deferred.matches.push('');
+            ignorePlugins.push('');
+        }
+        if (isDisabled('@jupyterlab/vdom-extension')) {
+            disabled.matches.push('@jupyterlab/vdom-extension');
+        } else {
+            var module = require('@jupyterlab/vdom-extension/');
+            var extension = module.default;
+
+            // Handle CommonJS exports.
+            if (!module.hasOwnProperty('__esModule')) {
+              extension = module;
+            }
+
+            if (Array.isArray(extension)) {
+                extension.forEach(function(plugin) {
+                    if (isDeferred(plugin.id)) {
+                        deferred.matches.push(plugin.id);
+                        ignorePlugins.push(plugin.id);
+                    }
+                    if (isDisabled(plugin.id)) {
+                        disabled.matches.push(plugin.id);
+                        return;
+                    }
+                    mimeExtensions.push(plugin);
+                });
+            } else {
+                mimeExtensions.push(extension);
+            }
+        }
+    } catch (e) {
+        console.error(e);
+    }
     try {
         if (isDeferred('')) {
             deferred.matches.push('');
