@@ -7,12 +7,10 @@ import os
 
 from jupyterlab_launcher import add_handlers, LabConfig
 from notebook.utils import url_path_join as ujoin
-from notebook.base.handlers import FileFindHandler
-from tornado.ioloop import IOLoop
 
 from .commands import (
-    get_app_dir, list_extensions, should_build, get_user_settings_dir, watch,
-    build
+    get_app_dir, build_check, get_user_settings_dir, watch,
+    build, ensure_dev_build
 )
 
 from .build_handler import build_path, Builder, BuildHandler
@@ -25,10 +23,9 @@ from ._version import __version__
 DEV_NOTE_NPM = """You're running JupyterLab from source.
 If you're working on the TypeScript sources of JupyterLab, try running
 
-    npm run watch
+    jupyter lab --dev-mode --watch
 
-from the JupyterLab repo directory in another terminal window to have the
-system incrementally watch and build JupyterLab's TypeScript for you, as you
+to have the system incrementally watch and build JupyterLab for you, as you
 make changes.
 """
 
@@ -74,9 +71,10 @@ def load_jupyter_server_extension(nbapp):
         core_mode = True
         config.settings_dir = ''
 
-    web_app.settings.setdefault('page_config_data', dict())
-    web_app.settings['page_config_data']['buildAvailable'] = True
-    web_app.settings['page_config_data']['token'] = nbapp.token
+    page_config = web_app.settings.setdefault('page_config_data', dict())
+    page_config['buildAvailable'] = not core_mode
+    page_config['buildCheck'] = not core_mode
+    page_config['token'] = nbapp.token
 
     if core_mode:
         config.assets_dir = os.path.join(here, 'build')
@@ -88,7 +86,7 @@ def load_jupyter_server_extension(nbapp):
             sentinel = os.path.join(here, 'build', 'release_data.json')
             config.dev_mode = not os.path.exists(sentinel)
 
-    if config.dev_mode:
+    if config.dev_mode and not watch_mode:
         nbapp.log.info(DEV_NOTE_NPM)
     elif core_mode:
         nbapp.log.info(CORE_NOTE.strip())
@@ -104,12 +102,14 @@ def load_jupyter_server_extension(nbapp):
     config.user_settings_dir = get_user_settings_dir()
 
     if watch_mode:
-        if core_mode:
-            watch(here, nbapp.log)
+        if config.dev_mode:
+            ensure_dev_build(nbapp.log)
+            watch(os.path.dirname(here), nbapp.log)
         else:
             config.assets_dir = os.path.join(app_dir, 'staging', 'build')
-            build(app_dir=app_dir, logger=nbapp.log)
+            build(app_dir=app_dir, logger=nbapp.log, command='build')
             watch(os.path.join(app_dir, 'staging'), nbapp.log)
+            page_config['buildAvailable'] = False
 
     add_handlers(web_app, config)
 
